"""Module contains the Student class, related helper functions, and test code."""

from dataclasses import dataclass
from is430_unit_test_helpers import assert_equal, assert_equal_float


@dataclass
class Student:
    """A class used to represent a student"""
    name:           str
    hours:          float
    quality_points: float

    def calculate_gpa(self):
        """
        calculates a GPA for this student

        :returns: gpa
        :rtype:   float
        """
        gpa = self.quality_points / self.hours
        return gpa


def make_student(student_record_text):
    """
    creates an instance of Student

    :param str student_record_text: tab-delimited string including name, hours, quality points.
    :returns: student_instance
    :rtype:   Student
    """
    name, hours_as_string, quality_points_as_string = student_record_text.split('\t')
    student_instance = Student(name, float(hours_as_string), float(quality_points_as_string))
    return student_instance


def main():
    print('Unit testing output follows...')

    # configure common test case variables
    expected_name = 'Adams, Henry'
    expected_hours = 127.0
    expected_quality_points = 228.0

    print('\nTest Case #1: Test constructor')
    student1 = Student(expected_name, expected_hours, expected_quality_points)
    assert_equal(expected_name, student1.name)
    assert_equal_float(expected_hours, student1.hours, 0.005)
    assert_equal_float(expected_quality_points, student1.quality_points, 0.005)

    print('\nTest Case #2: Test calculate_gpa()')
    student1 = Student(expected_name, expected_hours, expected_quality_points)
    expected_gpa = 1.7953
    actual_gpa = student1.calculate_gpa()
    assert_equal_float(expected_gpa, actual_gpa, 0.00005)

    print('\nTest Case #3: Test make_student()')
    student_record_text = f'{expected_name}\t{expected_hours}\t{expected_quality_points}\n'
    student2 = make_student(student_record_text)
    assert_equal(expected_name, student2.name)
    assert_equal(expected_hours, student2.hours)
    assert_equal(expected_quality_points, student2.quality_points)


if __name__ == '__main__':
    main()
